<?php
namespace Sharplogicians\Keycloak\Controller\Account;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Customer\Model\Session as CustomerSession;
use Magento\Framework\Stdlib\CookieManagerInterface;
use Sharplogicians\Keycloak\Model\Keycloak;
use Magento\Customer\Model\CustomerFactory;

class LoginCheck extends Action
{
    /**
     * @var CustomerSession
     */
    protected $customerSession;

    public function __construct(
        Context $context,
        CustomerSession $customerSession,
        CustomerFactory $customerFactory,
        Keycloak $keycloak,
        CookieManagerInterface $cookieManager
    ) {
        $this->customerSession = $customerSession;
        $this->customerFactory = $customerFactory;
        $this->keycloak = $keycloak;
        $this->cookieManager = $cookieManager;
        parent::__construct($context);
    }

    public function execute()
    {
        $accessToken = $this->cookieManager->getCookie('keycloak_access_token');

        if ($this->customerSession->isLoggedIn()) {
            if(!$accessToken) {
                $this->customerSession->logout();
                return;
            }
            $userInfo = $this->keycloak->getUserInfo($accessToken);            

            if (!isset($userInfo['email'])) {
                $this->customerSession->logout();
                return;
            }
            $customerId = $this->customerSession->getCustomerId();
            $customer = $this->customerFactory->create()->load($customerId);
            if($userInfo['email'] != $customer->getEmail()) {
                $customer = $this->keycloak->getOrCreateCustomer($userInfo['email'], $userInfo['name']);
                $customer = $this->customerFactory->create()->load($customer->getId());
                $this->customerSession->setCustomerAsLoggedIn($customer);
                return;
            }
        } else {
            if(!$accessToken) {
                return;
            }
            $userInfo = $this->keycloak->getUserInfo($accessToken);

            if (!isset($userInfo['email'])) {
                return;
            }

            $customer = $this->keycloak->getOrCreateCustomer($userInfo['email'], $userInfo['name']);
            $customer = $this->customerFactory->create()->load($customer->getId());
            $this->customerSession->setCustomerAsLoggedIn($customer);
            return;
        }        
    }
}
