<?php
namespace Sharplogicians\Keycloak\Model;

use Sharplogicians\Keycloak\Api\CustomerInterface;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Model\CustomerFactory;
use Magento\Customer\Model\AccountManagement;
use Magento\Framework\Exception\AuthenticationException;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Customer\Model\ResourceModel\Customer as CustomerResource;
use Sharplogicians\Keycloak\Model\Keycloak;
use Psr\Log\LoggerInterface;

class Customer implements CustomerInterface
{
    protected $customerRepository;

    protected $customerFactory;

    protected $tokenFactory;

    protected $encryptor;

    protected $customerResource;

    protected $keycloak;

    protected $logger;

    public function __construct(
        CustomerRepositoryInterface $customerRepository,
        CustomerFactory $customerFactory,
        AccountManagement $accountManagement,
        EncryptorInterface $encryptor,
        CustomerResource $customerResource,
        Keycloak $keycloak,
        LoggerInterface $logger
    ) {
        $this->customerRepository = $customerRepository;
        $this->customerFactory = $customerFactory;
        $this->accountManagement = $accountManagement;
        $this->encryptor = $encryptor;
        $this->customerResource = $customerResource;
        $this->keycloak = $keycloak;
        $this->logger = $logger;
    }

    public function authenticate($email, $password)
    {
        try {
            $rawPassword = $password;
            $customer = $this->accountManagement->authenticate($email, $password);
            $accessToken = $this->keycloak->authenticateUser($email, $rawPassword);

            echo json_encode([
                'success' => true,
                'keycloak_access_token' => $accessToken,
                'firstname' => $customer->getFirstname(),
                'lastname' => $customer->getLastname(),
                'email' => $customer->getEmail()
            ]);
        } catch (\Exception $e) {
            echo json_encode(['success' => false, 'message' => $e->getMessage()]);
        }
        exit;
    }
}
