<?php

namespace Sharplogicians\Keycloak\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Sharplogicians\Keycloak\Model\Keycloak;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Framework\Stdlib\CookieManagerInterface;
use Magento\Framework\Stdlib\Cookie\PublicCookieMetadata;
use Psr\Log\LoggerInterface;

class KeycloakLogin implements ObserverInterface
{
    private $customerRepository;

    private $keycloak;

    private $encryptor;

    protected $cookieManager;

    protected $_cookieMetadataFactory;

    protected $_sessionManager;

    protected $logger;

    public function __construct(
        CustomerRepositoryInterface $customerRepository,
        Keycloak $keycloak,
        EncryptorInterface $encryptor,
        CookieManagerInterface $cookieManager,
        \Magento\Framework\Stdlib\Cookie\CookieMetadataFactory $cookieMetadataFactory,
        \Magento\Framework\Session\SessionManagerInterface $sessionManager,
        LoggerInterface $logger
    ) {
        $this->customerRepository = $customerRepository;
        $this->keycloak = $keycloak;
        $this->encryptor = $encryptor;
        $this->cookieManager = $cookieManager;
        $this->_cookieMetadataFactory = $cookieMetadataFactory;
        $this->_sessionManager = $sessionManager;
        $this->logger = $logger;
    }

    public function execute(Observer $observer)
    {
        $customer = $observer->getData('model');
        $email = $customer->getEmail();
        $rawPassword = $observer->getData('password');
        $password = $this->encryptor->encrypt($rawPassword);
        $customer = $this->customerRepository->getById($customer->getId());
        $encryptedPassword = $customer->getCustomAttribute('encrypted_password') ? $customer->getCustomAttribute('encrypted_password')->getValue() : '';
        $synced = $customer->getCustomAttribute('synced') ? $customer->getCustomAttribute('synced')->getValue() : '';

        $adminToken = $this->keycloak->getAccessToken();
        if (!$synced) {
            $this->keycloak->exportCustomer($customer, $adminToken, $rawPassword);
            $customer->setCustomAttribute('synced', 1);
            $customer->setCustomAttribute('encrypted_password', $password);
        }
        $accessToken = $this->keycloak->authenticateUser($email, $rawPassword);
        $this->logger->info("Token: ".$accessToken);
        $this->setAccessTokenCookie($accessToken);
        
        $this->customerRepository->save($customer);
    }

    private function setAccessTokenCookie($accessToken)
    {
        $publicCookieMetadata = $this->_cookieMetadataFactory->createPublicCookieMetadata()
            ->setDuration(3600)
            ->setPath($this->_sessionManager->getCookiePath())
            ->setDomain($this->_sessionManager->getCookieDomain())
            ->setHttpOnly(false);
        $this->cookieManager->setPublicCookie('keycloak_access_token', $accessToken, $publicCookieMetadata);
    }
}
