<?php
namespace Sharplogicians\Keycloak\Observer;

use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Stdlib\CookieManagerInterface;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;
use Sharplogicians\Keycloak\Model\Keycloak;

class KeycloakLogout implements ObserverInterface
{
    private $keycloak;

    protected $cookieManager;

    protected $cookieMetadataFactory;

    private $cookieMetadataManager;

    protected $_sessionManager;

    public function __construct(
        Keycloak $keycloak,
        CookieManagerInterface $cookieManager,
        CookieMetadataFactory $cookieMetadataFactory,
        \Magento\Framework\Session\SessionManagerInterface $sessionManager
    ) {
        $this->keycloak = $keycloak;
        $this->cookieManager = $cookieManager;
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->_sessionManager = $sessionManager;
    }

    public function execute(Observer $observer)
    {
        if ($accessToken = $this->cookieManager->getCookie('keycloak_access_token')) {
            $this->keycloak->logoutUser($accessToken);
            $publicCookieMetadata = $this->cookieMetadataFactory->createPublicCookieMetadata()
                ->setDuration(3600)
                ->setPath($this->_sessionManager->getCookiePath())
                ->setDomain($this->_sessionManager->getCookieDomain())
                ->setHttpOnly(false);
            $this->cookieManager->setPublicCookie('keycloak_access_token', '', $publicCookieMetadata);
        }
    }
}
