<?php

namespace Sharplogicians\Keycloak\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Stdlib\CookieManagerInterface;
use Sharplogicians\Keycloak\Model\Keycloak;
use Magento\Customer\Model\CustomerFactory;

class KeycloakSession implements ObserverInterface
{
    private $customerSession;

    private $keycloak;

    private $cookieManager;

    public function __construct(
        \Magento\Customer\Model\Session $customerSession,
        CustomerFactory $customerFactory,
        Keycloak $keycloak,
        CookieManagerInterface $cookieManager
    ) {
        $this->customerSession = $customerSession;
        $this->customerFactory = $customerFactory;
        $this->keycloak = $keycloak;
        $this->cookieManager = $cookieManager;
    }

    public function execute(Observer $observer)
    {
        if ($this->customerSession->isLoggedIn()) {
            return;
        }

        try {
            $accessToken = $this->cookieManager->getCookie('keycloak_access_token');
            if(!$accessToken) {
                return;
            }
            $userInfo = $this->keycloak->getUserInfo($accessToken);

            if (!isset($userInfo['email'])) {
                return;
            }

            $customer = $this->keycloak->getOrCreateCustomer($userInfo['email'], $userInfo['name']);
            $customer = $this->customerFactory->create()->load($customer->getId());
            $this->customerSession->setCustomerAsLoggedIn($customer);
            $observer->getControllerAction()->getResponse()->setRedirect('/customer/account');
        } catch (\Exception $e) {
            
        }
    }
}
